/*
 * Copyright (C) 1997-1998 Kai 'Oswald' Seidler <oswald@cs.tu-berlin.de>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <gdbm.h>
#include <ctype.h>
#include <strings.h>

#ifdef SYSTIME
#include <sys/time.h>
#endif

#include "service.h"

#include "noteserv.h"

extern struct Clients *List_OnlineUsers[];
extern struct Clients *List_Subscriber;
extern int Service_Burst;

GDBM_FILE Last_FILE;

void User_Last(int argc, char *argv[])
{
    struct Clients *client;
    char *nickname = argv[0] + 1, *ptr, *ptra, *ptrb;
    char buffer[SMALLBUFFER];
    int dummy;
    int i;
    struct Last *last;

    if (!(client = List_find(&List_OnlineUsers[(int) nickname[0]], nickname, FIND_NICKNAME)))
	return;

    if (!(ptr = strchr(argv[4], '!')))
    {
	Client_sendNotice(client, 83, argv[4]);
	return;
    }
    if (!(ptra = strchr(ptr, '@')))
    {
	Client_sendNotice(client, 83, argv[4]);
	return;
    }
    ptrb = strchr(ptr, '*');
    if (ptrb < ptra || !ptrb)
    {
	Client_sendNotice(client, 83, argv[4]);
	return;
    }
    Time_stop();
    dummy = 0;
    i=0;
    while ((last = Last_get(argv[4], &dummy)) != 0)
    {
	strftime(buffer, SMALLBUFFER, "%c", gmtime(&(last->last)));
	Client_sendNotice(client, 85, last->key, buffer);
	dummy++;
	i++;
	if(i == MAX_FIND)
	{
		Client_sendNotice(client, 16, MAX_FIND);
		break;
	}
    }
    Client_sendNotice(client, 5, Time_stop());
}

void Last_add(char *keystring)
{
    datum data, key;
    char tmpkey[KEYLEN];
    struct Last last, *dummy;
    int i = 0;

	return;

    /* log only german users at the moment */
    if(match("*.de",keystring))
	    return;

    last.last = time(NULL);
    strcpy(last.key, keystring);

    dummy = Last_get(keystring, &i);

    sprintf(tmpkey, "%s.%d", Last_getKey(keystring), i);

    key.dptr = tmpkey;
    key.dsize = strlen(tmpkey) + 1;

    data.dptr = (char *) &last;
    data.dsize = sizeof(struct Last);

    gdbm_store(Last_FILE, key, data, GDBM_REPLACE);
}

struct Last *Last_get(char *pattern, int *j)
{
    datum data, key;
    char tmpkey[KEYLEN];
    char *dbkey = Last_getKey(pattern);
    int i = *j;
    struct Last *last;

    while (1 == 1)
    {
	sprintf(tmpkey, "%s.%d", dbkey, i);

	key.dptr = tmpkey;
	key.dsize = strlen(tmpkey) + 1;

	data = gdbm_fetch(Last_FILE, key);
	if (data.dptr)
	{
	    last = (struct Last *) data.dptr;

	    if (!match(pattern, last->key))
	    {
		*j = i;
		return last;
	    }
	} else
	{
	    *j = i;
	    return NULL;
	}
	i++;
    }

}

void Last_Init()
{
    char buffer[MAXBUFFER];

    sprintf(buffer, "%s/%s", DIRECTORY, "lastlog");
    if (!(Last_FILE = gdbm_open(buffer, 512, GDBM_WRCREAT | GDBM_FAST, 0600, 0)))
    {
	perror("gdbm_open()");
	exit(13);
    }
    gdbm_reorganize(Last_FILE); 
}

void Last_Sync()
{
    gdbm_sync(Last_FILE);
}

void Last_Close()
{
    gdbm_close(Last_FILE);
}


char *Last_getKey(char *key)
{
    static char buffer[KEYLEN];
    char *m, *ptr, *lastdot;

    m = strchr(key, '!');
    key = m + 1;


    ptr = buffer;
    while ((*ptr++ = *key++) != '@');

    lastdot = strrchr(key, '.');
    if (lastdot[1] < 'a')	/* is the host part an ip address? */
    {
	while (key < lastdot)
	{
	    *ptr++ = *key++;
	}
    } else
    {

	key = strchr(key, '.') + 1;

	while ((*ptr++ = *key++) != '\0');

	*ptr = '\0';
    }

    return buffer;
}
